@@ -48,7 +48,7 @@ class Plugin(RestExtension):
         global _site
         _site = self.site = site
         directives.register_directive('chart', Chart)
-        return super(Plugin, self).set_site(site)
+        return super().set_site(site)
 
 
 class Chart(Directive):
