@@ -547,7 +547,7 @@ class Post(object):
             lang = nikola.utils.LocaleBorg().current_lang
         return self.meta[lang]['title']
 
-    def author(self, lang=None):
+    def author(self, lang=None) -> str:
         """Return localized author or BLOG_AUTHOR if unspecified.
 
         If lang is not specified, it defaults to the current language from
@@ -562,6 +562,21 @@ class Post(object):
 
         return author
 
+    def authors(self, lang=None) -> list:
+        """Return localized authors or BLOG_AUTHOR if unspecified.
+
+        If lang is not specified, it defaults to the current language from
+        templates, as set in LocaleBorg.
+        """
+        if lang is None:
+            lang = nikola.utils.LocaleBorg().current_lang
+        if self.meta[lang]['author']:
+            author = [i.strip() for i in self.meta[lang]['author'].split(",")]
+        else:
+            author = [self.config['BLOG_AUTHOR'](lang)]
+
+        return author
+
     def description(self, lang=None):
         """Return localized description."""
         if lang is None:
@@ -646,7 +661,7 @@ class Post(object):
             deps_path = post.compiler.get_dep_filename(post, lang)
         if deps_list or (post.compiler.use_dep_file if post else False):
             deps_list = [p for p in deps_list if p != dest]  # Don't depend on yourself (#1671)
-            with io.open(deps_path, "w+", encoding="utf8") as deps_file:
+            with io.open(deps_path, "w+", encoding="utf-8") as deps_file:
                 deps_file.write('\n'.join(deps_list))
         else:
             if os.path.isfile(deps_path):
@@ -843,7 +858,7 @@ class Post(object):
             lang = nikola.utils.LocaleBorg().current_lang
 
         source = self.translated_source_path(lang)
-        with open(source, encoding='utf-8') as inf:
+        with open(source, 'r', encoding='utf-8-sig') as inf:
             data = inf.read()
         if self.is_two_file:  # Metadata is not here
             source_data = data
@@ -874,7 +889,7 @@ class Post(object):
         if not os.path.isfile(file_name):
             self.compile(lang)
 
-        with io.open(file_name, "r", encoding="utf8") as post_file:
+        with io.open(file_name, "r", encoding="utf-8-sig") as post_file:
             data = post_file.read().strip()
 
         if self.compiler.extension() == '.php':
@@ -975,7 +990,7 @@ class Post(object):
             # duplicated with Post.text()
             lang = nikola.utils.LocaleBorg().current_lang
             file_name, _ = self._translated_file_path(lang)
-            with io.open(file_name, "r", encoding="utf8") as post_file:
+            with io.open(file_name, "r", encoding="utf-8-sig") as post_file:
                 data = post_file.read().strip()
             try:
                 document = lxml.html.fragment_fromstring(data, "body")
@@ -1070,11 +1085,13 @@ class Post(object):
 
         image_path = self.meta[lang]['previewimage']
         if not image_path:
-            return self._default_preview_image
+            image_path = self._default_preview_image
 
-        # This is further parsed by the template, because we don’t have access
-        # to the URL replacer here.  (Issue #1473)
-        return image_path
+        if not image_path or image_path.startswith("/"):
+            # Paths starting with slashes are expected to be root-relative, pass them directly.
+            return image_path
+        # Other paths are relative to the permalink. The path will be made prettier by the URL replacer later.
+        return urljoin(self.permalink(lang), image_path)
 
     def source_ext(self, prefix=False):
         """Return the source file extension.
